# sq

sq is a [jq][] wrapper for S-expressions.

Specifically, it round-trips [SRFI-180][]-formatted inputs through jq, allowing
you to use that program's filters and flags to manipulate Scheme data.

Think of it like [yq][], but for S-expressions instead of YAML.

[jq]: https://github.com/stedolan/jq
[yq]: https://github.com/kislyuk/yq

## Resources

 * Sources: <https://hg.sr.ht/~evhan/sq>
 * Issues: <https://todo.sr.ht/~evhan/sq>

## Installation

Use your distribution's package manager to install CHICKEN 5.2 or newer, then
run:

``` sh
chicken-install -s sq
```

Because sq calls jq internally, you will also need to install jq according to
the [download instructions][jq/download] for your platform.

[jq/download]: https://stedolan.github.io/jq/download/

## Usage

Invoke sq just like you would jq. It accepts either S-expressions or JSON as
input. When S-expressions are used, they must follow the format described by
[SRFI-180][], for which the mapping between JSON types and Scheme objects is
the following:

 - `null` → the symbol `null`
 - `true` → `#t`
 - `false` → `#f`
 - number → number
 - string → string
 - array → vector
 - object → association list with keys that are symbols

By default, sq generates S-expressions:

``` sh
$ echo '{"greeting": "hello"}'  | sq
((greeting . "hello"))
$ echo '((greeting . "hello"))' | sq
((greeting . "hello"))
```

To generate JSON instead, pass the `-j` or `--json` flag:

``` sh
$ echo '{"greeting": "hello"}'  | sq --json
{
  "greeting": "hello"
}
$ echo '((greeting . "hello"))' | sq --json
{
  "greeting": "hello"
}
```

Any other flags (apart from `--help` and `--version`) are passed directly
through to jq. For example, use the `-r` or `--raw-output` flag to output
string values:

``` sh
$ echo '{"greeting": "hello"}'  | sq --raw-output .greeting
hello
$ echo '((greeting . "hello"))' | sq --raw-output .greeting
hello
```

Note that not all flags will work with sq. For example, the `--color-output`
flag will only work when combined with `--json`, and the `--null-input` flag is
obviously useless since sq works by sending input to jq.

[SRFI-180]: https://srfi.schemers.org/srfi-180/srfi-180.html

## Known Problems

- Parsing is slow for large JSON outputs generated by jq.
- The program should probably just execute jq when `--json` is used with JSON inputs.
- There is no validation of command line flags for compatibility.
- Error messages for invalid S-expression inputs are not very helpful.
- There are no tests.

## License

Three-clause BSD. See LICENSE for details.
